# Example script from: https://github.com/alphacep/vosk-api/tree/master/python/example

import os
import json
import queue
import sounddevice as sd
import vosk
import sys

# ---
# CONFIGURATIONS

# Select your microphone
device = 18 # this is default on my computer
# You can find the number of your input device by printing sd.query_devices()
# print(sd.query_devices())

# ---
# QUEUE WITH INPUT DATA FROM THE MIC

q = queue.Queue()

# ---

def callback(indata, frames, time, status):
    """This is called (from a separate thread) for each audio block."""
    if status:
        print(status, file=sys.stderr)
    q.put(bytes(indata))

try:

    # Provide the name of the folder with the model
    model = "model"

    # Read the samplerate of your input device
    device_info = sd.query_devices(device, 'input') # soundfile expects an int, sounddevice provides a float:
    samplerate = int(device_info['default_samplerate'])

    # Load the output file
    if output_filename:
        output_file = open(output_filename, "wb")
    else:
        output_file = None

    # ---
    # LOADING THE VOSK MODEL

    model = vosk.Model(model)

    # ---
    # RECEIVING INPUT FROM THE MIC

    with sd.RawInputStream(samplerate=samplerate, blocksize = 8000, device=device, dtype='int16', channels=1, callback=callback):

            print('#' * 80, file=sys.stderr)
            print('Press Ctrl+C to stop the recording', file=sys.stderr)
            print('#' * 80, file=sys.stderr)

            # --- 
            # VOSK RECORDING OBJECT
            
            rec = vosk.KaldiRecognizer(model, samplerate)

            while True:
                
                # ---
                # THE CALLBACK SENDS THE INPUT DATA TO THE QUEUE
                # HERE, DATA IS TAKEN FROM THE QUEUE

                data = q.get()

                if rec.AcceptWaveform(data):

                    # ---
                    # PRINTING RESULTS AS PLAIN TEXT

                    output = json.loads(rec.Result())
                    print(output['text'], flush=True) 


except KeyboardInterrupt:
    
    print('\nDone', file=sys.stderr)
